# -*- coding: utf-8 -*-

import conf
import logging
import xmltodict
import lxml.etree
import lxml.builder
from django.http import HttpResponse
from django.views.generic import View, TemplateView
from annoying.functions import get_object_or_None
from django.views.decorators.csrf import csrf_exempt
from .utils import get_signature
from .models import Payment

logger = logging.getLogger('pay2pay')


class Confirm(View):
    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        return super(Confirm, self).dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        error_msg = ''

        xml = request.POST.get('xml', '').replace(' ', '+').decode('base64')
        try:
            response = self._get_obj_response(xml)
        except (ValueError, TypeError):
            response = {}
            logger.error('Can not parse xml', exc_info=True)

        payment = get_object_or_None(Payment, order_id=response.get('order_id'))
        if payment:
            sig_encode = request.POST.get('sign', '')
            sign = get_signature(xml, conf.PAY2PAY_HIDE_KEY)
            if sig_encode == sign:
                try:
                    payment.status = response['status']
                    payment.paymode = response['paymode']
                    payment.trans_id = response['trans_id']
                    payment.error_msg = response.get('error_msg', '')
                    if response.get('test_mode', '') == '1':
                        payment.test_mode = True
                    payment.save()
                    payment.send_signals()
                except KeyError:
                    logger.error('Have not some key at parsed XML data', exc_info=True)
            else:
                error_msg = 'Security check failed'
        else:
            error_msg = 'Unknown order_id'

        if error_msg:
            logger.error(error_msg, exc_info=True)

        xml_response = self._get_xml_response(error_msg)
        return HttpResponse(xml_response)

    def _get_xml_response(self, error_msg_value):
        e = lxml.builder.ElementMaker()
        request = e.request
        status = e.status
        error_msg = e.error_msg

        if error_msg_value:
            status_value = 'no'
        else:
            status_value = 'yes'

        request = request(
            status(status_value),
            error_msg(error_msg_value)
        )
        return lxml.etree.tostring(request, encoding='utf-8').replace('\n', '')

    def _get_obj_response(self, xml):
        xml = xml.replace('<?xmlversion="1.0"encoding="UTF-8"?>', '').replace('\n', '')
        return xmltodict.parse(xml)['response']


class PaymentSuccess(TemplateView, Confirm):
    template_name = 'pay2pay/payment_success.html'

    def post(self, request, *args, **kwargs):
        super(PaymentSuccess, self).post(request, *args, **kwargs)
        return super(PaymentSuccess, self).get(request, *args, **kwargs)


class PaymentFail(TemplateView, Confirm):
    template_name = 'pay2pay/payment_fail.html'

    def post(self, request, *args, **kwargs):
        super(PaymentFail, self).post(request, *args, **kwargs)
        return super(PaymentFail, self).get(request, *args, **kwargs)


# -*- coding: utf-8 -*-

from mock import patch
from django.conf import settings
from django.test import TestCase
from django_webtest import WebTest
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from .utils import build_xml_string
from .utils import get_signature
from .factories import PaymentF
from .models import Payment


class UtilsTest(TestCase):
    def setUp(self):
        self.xml = '<request><version>1.3</version><merchant_id>2669</merchant_id><order_id>0143ef7d</order_id><amount>1.0</amount><currency>RUB</currency><description>Описание заказа</description><success_url>http://localhost:8000/success/</success_url><fail_url>http://localhost:8000/fail/</fail_url><result_url>http://localhost:8000/success/</result_url><test_mode>1</test_mode></request>'

    def test_build_xml_string(self):
        data = {
            'test_mode': 1,
            'version': u'1.3',
            'order_id': '0143ef7d',
            'fail_url': 'http://localhost:8000/fail/',
            'currency': u'RUB',
            'amount': 1.0,
            'result_url': 'http://localhost:8000/success/',
            'success_url': 'http://localhost:8000/success/',
            'merchant_id': 2669,
            'description': u'\u041e\u043f\u0438\u0441\u0430\u043d\u0438\u0435 \u0437\u0430\u043a\u0430\u0437\u0430'
        }
        xml = build_xml_string(data)

        self.assertEquals(self.xml, xml, 'XML built is wrong')

    @override_settings(PAY2PAY_SECRET_KEY='qCmm7dsaSdasfsqCmgdjfgkdfghdfsad')
    def test_get_signature(self):
        sign_encode = get_signature(self.xml, settings.PAY2PAY_SECRET_KEY)
        self.assertEquals('MzVlMDJiZmFlODU4ZDM2MWNiZDJkZGM0ZmRkMzQ0OWY=', sign_encode, 'XML not signed')


class OrderModelTest(TestCase):
    @patch('uuid.uuid4', lambda: 'ea2fe31e')
    def test_get_xml_data(self):
        test_data = {
            'version': u'1.3',
            'order_id': 'ea2fe31e',
            'fail_url': 'http://localhost:8000/fail/',
            'currency': u'RUB',
            'result_url': 'http://localhost:8000/success/',
            'success_url': 'http://localhost:8000/success/',
            'merchant_id': 2669,
        }

        payment = Payment()
        data = payment._get_xml_data()

        self.assertEquals(test_data, data, 'Data from Payment model was wrong')


class PaymentTestBase(WebTest):
    named_url = ''
    order_id='24e59062-7388-4f'

    def setUp(self):
        self.xml = '<?xml version="1.0" encoding="UTF-8"?><response><version>1.3</version><type>result</type><merchant_id>2669</merchant_id><language></language><order_id>24e59062-7388-4f</order_id><amount>1.00</amount><currency>RUB</currency><description>Описание заказа</description><paymode>bank</paymode><trans_id>395156</trans_id><status>success</status><error_msg></error_msg><test_mode>1</test_mode><other><![CDATA[]]></other></response>'
        self.xml_encode = 'PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVVRGLTgiPz48cmVzcG9uc2U PHZlcnNpb24 MS4zPC92ZXJzaW9uPjx0eXBlPnJlc3VsdDwvdHlwZT48bWVyY2hhbnRfaWQ MjY2OTwvbWVyY2hhbnRfaWQ PGxhbmd1YWdlPjwvbGFuZ3VhZ2U PG9yZGVyX2lkPjI0ZTU5MDYyLTczODgtNGY8L29yZGVyX2lkPjxhbW91bnQ MS4wMDwvYW1vdW50PjxjdXJyZW5jeT5SVUI8L2N1cnJlbmN5PjxkZXNjcmlwdGlvbj7QntC/0LjRgdCw0L3QuNC1INC30LDQutCw0LfQsDwvZGVzY3JpcHRpb24 PHBheW1vZGU YmFuazwvcGF5bW9kZT48dHJhbnNfaWQ Mzk1MTU2PC90cmFuc19pZD48c3RhdHVzPnN1Y2Nlc3M8L3N0YXR1cz48ZXJyb3JfbXNnPjwvZXJyb3JfbXNnPjx0ZXN0X21vZGU MTwvdGVzdF9tb2RlPjxvdGhlcj48IVtDREFUQVtdXT48L290aGVyPjwvcmVzcG9uc2U '

        payment = PaymentF(order_id=self.order_id)
        payment.save()

        self.url = reverse(self.named_url)

    def tearDown(self):
        Payment.objects.get(order_id=self.order_id)

    def _get_obj_response(self, xml):
        return {
            'status': 'success',
            'test_mode': '1',
            'description': u'Описание',
            'order_id': '24e59062-7388-4f',
            'error_msg': '',
            'paymode': 'Paymode_Code',
            'currency': 'RUB',
            'amount': 1.0,
            'version': '1.3',
            'merchant_id': 2669,
            'trans_id': '394556',
        }

    def _get_signature(self, key):
        return 'NDU2ZmUzYTExNzA3ZjA5NDQ3NmQxM2E0YTY5YzEwZWM='

    @override_settings(PAY2PAY_HIDE_KEY='qCmm7SNTSdasfsqCmm7SNTSd')
    @patch('pay2pay.views.Confirm._get_obj_response', _get_obj_response)
    @patch('pay2pay.utils.get_signature', _get_signature)
    def test_update_status(self):
        sign_encode = get_signature(self.xml, settings.PAY2PAY_HIDE_KEY)

        params = {
            'xml': self.xml_encode,
            'sign': sign_encode,
        }
        self.app.post(self.url, params=params)

        payment = Payment.objects.get(order_id=self.order_id)
        self.assertEquals(payment.status, 'success', 'Payment status was not updated')


class ConfirmTest(PaymentTestBase):
    named_url = 'pay2pay_confirm'

    @override_settings(PAY2PAY_HIDE_KEY='qCmm7SNTSdasfsqCmm7SNTSd')
    def test_response(self):
        sign_encode = get_signature(self.xml, settings.PAY2PAY_HIDE_KEY)

        params = {
            'xml': self.xml_encode,
            'sign': sign_encode,
        }
        response = self.app.post(self.url, params=params)

        self.assertEquals(200, response.status_code, 'Confirm view is unavalible')
        self.assertIn('<status>yes</status>', response.text, 'Response has not status yes')
        self.assertIn('<error_msg></error_msg>', response.text, 'Response have error message')


class SuccessTest(PaymentTestBase):
    named_url = 'pay2pay_success'


class FailTest(PaymentTestBase):
    named_url = 'pay2pay_fail'


# -*- coding: utf-8 -*-

import conf
import base64
from django import forms
from inspect import isfunction
from .utils import get_signature
from .utils import build_xml_string
from .models import Payment


class PayForm(forms.ModelForm):
    xml = forms.CharField()
    sign = forms.CharField()

    def __init__(self, *args, **kwargs):
        super(PayForm, self).__init__(*args, **kwargs)

        data = {
            'success_url': conf.PAY2PAY_SUCCESS_URL,
            'fail_url': conf.PAY2PAY_FAIL_URL,
            'result_url': conf.PAY2PAY_RESULT_URL,
        }
        if conf.PAY2PAY_TEST_MODE:
            data['test_mode'] = 1
        initial = kwargs.get('initial', {})
        for name in Payment.names:
            if name in initial:
                data[name] = initial[name]
            else:
                default = self.fields[name].initial
                if isfunction(default):
                    data[name] = default()
                else:
                    data[name] = default

        xml = build_xml_string(data)
        self.fields['xml'].initial = base64.b64encode(xml)
        self.fields['sign'].initial = get_signature(xml, conf.PAY2PAY_SECRET_KEY)

        if conf.PAY2PAY_HIDE_FORM_FIELD:
            for name in self.fields:
                self.fields[name].widget = forms.HiddenInput()

    class Meta:
        exclude = ('created',)
        model = Payment


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Payment'
        db.create_table(u'pay2pay_payment', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('version', self.gf('django.db.models.fields.CharField')(default='1.3', max_length=8)),
            ('merchant_id', self.gf('django.db.models.fields.PositiveIntegerField')(default=2669)),
            ('order_id', self.gf('django.db.models.fields.CharField')(default='a4f3990ab2ae4c96', max_length=16)),
            ('amount', self.gf('django.db.models.fields.FloatField')(default=0.0)),
            ('currency', self.gf('django.db.models.fields.CharField')(default='RUB', max_length=8)),
            ('description', self.gf('django.db.models.fields.CharField')(default='', max_length=512)),
            ('paymode', self.gf('django.db.models.fields.CharField')(max_length=16, null=True, blank=True)),
            ('trans_id', self.gf('django.db.models.fields.CharField')(max_length=32, null=True, blank=True)),
            ('status', self.gf('django.db.models.fields.CharField')(default='not_send', max_length=16)),
            ('error_msg', self.gf('django.db.models.fields.CharField')(max_length=256, null=True, blank=True)),
            ('test_mode', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'pay2pay', ['Payment'])


    def backwards(self, orm):
        # Deleting model 'Payment'
        db.delete_table(u'pay2pay_payment')


    models = {
        u'pay2pay.payment': {
            'Meta': {'ordering': "('created',)", 'object_name': 'Payment'},
            'amount': ('django.db.models.fields.FloatField', [], {'default': '0.0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'currency': ('django.db.models.fields.CharField', [], {'default': "'RUB'", 'max_length': '8'}),
            'description': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '512'}),
            'error_msg': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'merchant_id': ('django.db.models.fields.PositiveIntegerField', [], {'default': '2669'}),
            'order_id': ('django.db.models.fields.CharField', [], {'default': "'3be556531c604af2'", 'max_length': '16'}),
            'paymode': ('django.db.models.fields.CharField', [], {'max_length': '16', 'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'not_send'", 'max_length': '16'}),
            'test_mode': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'trans_id': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'version': ('django.db.models.fields.CharField', [], {'default': "'1.3'", 'max_length': '8'})
        }
    }

    complete_apps = ['pay2pay']


# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url
from .views import Confirm
from .views import PaymentFail
from .views import PaymentSuccess

urlpatterns = patterns('',
    url(r'^confirm/$', Confirm.as_view(), name='pay2pay_confirm'),
    url(r'^success/$', PaymentFail.as_view(), name='pay2pay_success'),
    url(r'^fail/$', PaymentSuccess.as_view(), name='pay2pay_fail'),
)


# -*- coding: utf-8 -*-

from django.conf import settings


PAY2PAY_MERCHANT_ID = getattr(settings, 'PAY2PAY_MERCHANT_ID')
PAY2PAY_HIDE_KEY = getattr(settings, 'PAY2PAY_HIDE_KEY')
PAY2PAY_SECRET_KEY = getattr(settings, 'PAY2PAY_SECRET_KEY')
PAY2PAY_CURRENCY = getattr(settings, 'PAY2PAY_CURRENCY', 'RUB')
PAY2PAY_SUCCESS_URL = getattr(settings, 'PAY2PAY_SUCCESS_URL')
PAY2PAY_FAIL_URL = getattr(settings, 'PAY2PAY_FAIL_URL')
PAY2PAY_RESULT_URL = getattr(settings, 'PAY2PAY_RESULT_URL')
PAY2PAY_TEST_MODE = getattr(settings, 'PAY2PAY_TEST_MODE', False)
PAY2PAY_HIDE_FORM_FIELD = getattr(settings, 'PAY2PAY_HIDE_FORM_FIELD', True)



# -*- coding: utf-8 -*-

import base64
import hashlib
import lxml.etree
import lxml.builder


def build_xml_string(data):
    e = lxml.builder.ElementMaker()

    request = e.request
    version = e.version
    merchant_id = e.merchant_id
    order_id = e.order_id
    amount = e.amount
    currency = e.currency
    description = e.description
    success_url = e.success_url
    fail_url = e.fail_url
    result_url = e.result_url
    test_mode = e.test_mode

    request = request(
        version(data['version']),
        merchant_id(str(data['merchant_id'])),
        order_id(data['order_id']),
        amount(str(data['amount'])),
        currency(data['currency']),
        description(data['description']),
        success_url(data['success_url']),
        fail_url(data['fail_url']),
        result_url(data['result_url']),
        test_mode(
            str(data.get('test_mode', ''))
        ),
    )
    return lxml.etree.tostring(request, encoding='utf-8').replace('\n', '')


def get_signature(xml, key):
    sign_str = '{0}{1}{0}'.format(key, xml)
    hsh = hashlib.md5()
    hsh.update(sign_str)
    sign_md5 = hsh.hexdigest()
    sign_encode = base64.b64encode(sign_md5)
    return sign_encode


# -*- coding: utf-8 -*-

from django.contrib import admin
from .models import Payment


class PaymentAdmin(admin.ModelAdmin):
    list_display = ('order_id', 'amount', 'currency', 'paymode', 'status', 'trans_id', 'description', 'created')
    list_filter = ('merchant_id', 'paymode', 'status', 'created')
    search_fields = ('order_id', 'description', 'error_msg')

admin.site.register(Payment, PaymentAdmin)


# -*- coding: utf-8 -*-

import conf
import uuid
from django.db import models
from .signals import payment_process
from .signals import payment_completed
from .signals import payment_fail
from .utils import build_xml_string
from .utils import get_signature


class Payment(models.Model):
    STATUS_NOT_SEND = 'not_send'
    STATUS_PROCESS = 'process'
    STATUS_RESERVE = 'reserve'
    STATUS_SUCCESS = 'success'
    STATUS_FAIL = 'fail'
    STATUS_CHOICES = (
        (STATUS_NOT_SEND, 'Не отправлен'),
        (STATUS_PROCESS, 'Ожидается оплата'),
        (STATUS_RESERVE, 'Средства зарезервированы'),
        (STATUS_SUCCESS, 'заказ оплачен'),
        (STATUS_FAIL, 'Оплата отменена'),
    )
    version = models.CharField('Версия интерфейса', max_length=8, default='1.3')
    merchant_id = models.PositiveIntegerField('ID магазина', default=conf.PAY2PAY_MERCHANT_ID)
    order_id = models.CharField('Номер заказа', max_length=16, default=lambda: str(uuid.uuid4()).replace('-', '')[:16])
    amount = models.FloatField('Сумма', default=.0)
    currency = models.CharField('Валюта', max_length=8, default=conf.PAY2PAY_CURRENCY)
    description = models.CharField('Описание', max_length=512, default='')

    paymode = models.CharField('Способ платежа', max_length=16, blank=True, null=True)
    trans_id = models.CharField('Номер транзакции', max_length=32, blank=True, null=True)
    status = models.CharField('Статус', max_length=16, choices=STATUS_CHOICES, default=STATUS_NOT_SEND)
    error_msg = models.CharField('Описание ошибки', max_length=256, blank=True, null=True)
    test_mode = models.BooleanField('Тестовый режим', default=conf.PAY2PAY_TEST_MODE)

    updated = models.DateTimeField('Обновлен', auto_now=True)
    created = models.DateTimeField('Создан', auto_now_add=True)

    def __unicode__(self):
        return '%s <%s>' % (
            self.order_id,
            self.get_status_display()
        )

    def send_signals(self):
        if not self.status:
            return

        status = self.status
        if status == self.STATUS_PROCESS or status == self.STATUS_RESERVE:
            payment_process.send(sender=self)
        if status == self.STATUS_SUCCESS:
            payment_completed.send(sender=self)
        if status == self.STATUS_FAIL:
            payment_fail.send(sender=self)

    @property
    def signature(self):
        return get_signature(self.xml, conf.PAY2PAY_SECRET_KEY)

    @property
    def xml(self):
        data = self._get_xml_data()
        return build_xml_string(data)

    def _get_xml_data(self):
        data = {
            'success_url': conf.PAY2PAY_SUCCESS_URL,
            'fail_url': conf.PAY2PAY_FAIL_URL,
            'result_url': conf.PAY2PAY_RESULT_URL,
        }
        if conf.PAY2PAY_TEST_MODE:
            data['test_mode'] = 1
        for name in self.names:
            value = getattr(self, name)
            if value:
                data[name] = value

        return data

    names = [
        'version',
        'merchant_id',
        'order_id',
        'amount',
        'currency',
        'description',
    ]

    class Meta:
        ordering = ('created',)
        verbose_name = 'Заказ'
        verbose_name_plural = 'Заказы'